@@ -5,6 +5,7 @@ import itertools
 import operator
 from typing import (
     Any,
+    Callable,
     cast,
 )
 import warnings
@@ -16,12 +17,13 @@ from pandas._config import get_option
 from pandas._libs import (
     NaT,
     NaTType,
-    Timedelta,
     iNaT,
     lib,
 )
 from pandas._typing import (
     ArrayLike,
+    AxisInt,
+    CorrelationMethod,
     Dtype,
     DtypeObj,
     F,
@@ -30,6 +32,7 @@ from pandas._typing import (
     npt,
 )
 from pandas.compat._optional import import_optional_dependency
+from pandas.util._exceptions import find_stack_level
 
 from pandas.core.dtypes.common import (
     is_any_int_dtype,
@@ -72,7 +75,7 @@ set_use_bottleneck(get_option("compute.use_bottleneck"))
 
 
 class disallow:
-    def __init__(self, *dtypes: Dtype):
+    def __init__(self, *dtypes: Dtype) -> None:
         super().__init__()
         self.dtypes = tuple(pandas_dtype(dtype).type for dtype in dtypes)
 
@@ -104,7 +107,7 @@ class disallow:
 
 
 class bottleneck_switch:
-    def __init__(self, name=None, **kwargs):
+    def __init__(self, name=None, **kwargs) -> None:
         self.name = name
         self.kwargs = kwargs
 
@@ -120,7 +123,7 @@ class bottleneck_switch:
         def f(
             values: np.ndarray,
             *,
-            axis: int | None = None,
+            axis: AxisInt | None = None,
             skipna: bool = True,
             **kwds,
         ):
@@ -162,6 +165,10 @@ class bottleneck_switch:
 def _bn_ok_dtype(dtype: DtypeObj, name: str) -> bool:
     # Bottleneck chokes on datetime64, PeriodDtype (or and EA)
     if not is_object_dtype(dtype) and not needs_i8_conversion(dtype):
+        # GH 42878
+        # Bottleneck uses naive summation leading to O(n) loss of precision
+        # unlike numpy which implements pairwise summation, which has O(log(n)) loss
+        # crossref: https://github.com/pydata/bottleneck/issues/379
 
         # GH 15507
         # bottleneck does not properly upcast during the sum
@@ -171,13 +178,13 @@ def _bn_ok_dtype(dtype: DtypeObj, name: str) -> bool:
         # further we also want to preserve NaN when all elements
         # are NaN, unlike bottleneck/numpy which consider this
         # to be 0
-        return name not in ["nansum", "nanprod"]
+        return name not in ["nansum", "nanprod", "nanmean"]
     return False
 
 
 def _has_infs(result) -> bool:
     if isinstance(result, np.ndarray):
-        if result.dtype == "f8" or result.dtype == "f4":
+        if result.dtype in ("f8", "f4"):
             # Note: outside of an nanops-specific test, we always have
             #  result.ndim == 1, so there is no risk of this ravel making a copy.
             return lib.has_infs(result.ravel("K"))
@@ -364,22 +371,26 @@ def _wrap_results(result, dtype: np.dtype, fill_value=None):
                 result = np.nan
 
             if isna(result):
-                result = np.datetime64("NaT", "ns")
+                result = np.datetime64("NaT", "ns").astype(dtype)
             else:
-                result = np.int64(result).view("datetime64[ns]")
+                result = np.int64(result).view(dtype)
+            # retain original unit
+            result = result.astype(dtype, copy=False)
         else:
             # If we have float dtype, taking a view will give the wrong result
             result = result.astype(dtype)
     elif is_timedelta64_dtype(dtype):
         if not isinstance(result, np.ndarray):
-            if result == fill_value:
-                result = np.nan
+            if result == fill_value or np.isnan(result):
+                result = np.timedelta64("NaT").astype(dtype)
 
-            # raise if we have a timedelta64[ns] which is too large
-            if np.fabs(result) > lib.i8max:
+            elif np.fabs(result) > lib.i8max:
+                # raise if we have a timedelta64[ns] which is too large
                 raise ValueError("overflow in timedelta operation")
+            else:
+                # return a timedelta64 with the original unit
+                result = np.int64(result).astype(dtype, copy=False)
 
-            result = Timedelta(result, unit="ns")
         else:
             result = result.astype("m8[ns]").view(dtype)
 
@@ -396,7 +407,7 @@ def _datetimelike_compat(func: F) -> F:
     def new_func(
         values: np.ndarray,
         *,
-        axis: int | None = None,
+        axis: AxisInt | None = None,
         skipna: bool = True,
         mask: npt.NDArray[np.bool_] | None = None,
         **kwargs,
@@ -420,7 +431,7 @@ def _datetimelike_compat(func: F) -> F:
     return cast(F, new_func)
 
 
-def _na_for_min_count(values: np.ndarray, axis: int | None) -> Scalar | np.ndarray:
+def _na_for_min_count(values: np.ndarray, axis: AxisInt | None) -> Scalar | np.ndarray:
     """
     Return the missing value for `values`.
 
@@ -459,7 +470,7 @@ def maybe_operate_rowwise(func: F) -> F:
     """
 
     @functools.wraps(func)
-    def newfunc(values: np.ndarray, *, axis: int | None = None, **kwargs):
+    def newfunc(values: np.ndarray, *, axis: AxisInt | None = None, **kwargs):
         if (
             axis == 1
             and values.ndim == 2
@@ -488,7 +499,7 @@ def maybe_operate_rowwise(func: F) -> F:
 def nanany(
     values: np.ndarray,
     *,
-    axis: int | None = None,
+    axis: AxisInt | None = None,
     skipna: bool = True,
     mask: npt.NDArray[np.bool_] | None = None,
 ) -> bool:
@@ -509,16 +520,25 @@ def nanany(
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([1, 2])
     >>> nanops.nanany(s)
     True
 
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([np.nan])
     >>> nanops.nanany(s)
     False
     """
+    if needs_i8_conversion(values.dtype) and values.dtype.kind != "m":
+        # GH#34479
+        warnings.warn(
+            "'any' with datetime64 dtypes is deprecated and will raise in a "
+            "future version. Use (obj != pd.Timestamp(0)).any() instead.",
+            FutureWarning,
+            stacklevel=find_stack_level(),
+        )
+
     values, _, _, _, _ = _get_values(values, skipna, fill_value=False, mask=mask)
 
     # For object type, any won't necessarily return
@@ -534,7 +554,7 @@ def nanany(
 def nanall(
     values: np.ndarray,
     *,
-    axis: int | None = None,
+    axis: AxisInt | None = None,
     skipna: bool = True,
     mask: npt.NDArray[np.bool_] | None = None,
 ) -> bool:
@@ -555,16 +575,25 @@ def nanall(
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([1, 2, np.nan])
     >>> nanops.nanall(s)
     True
 
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([1, 0])
     >>> nanops.nanall(s)
     False
     """
+    if needs_i8_conversion(values.dtype) and values.dtype.kind != "m":
+        # GH#34479
+        warnings.warn(
+            "'all' with datetime64 dtypes is deprecated and will raise in a "
+            "future version. Use (obj != pd.Timestamp(0)).all() instead.",
+            FutureWarning,
+            stacklevel=find_stack_level(),
+        )
+
     values, _, _, _, _ = _get_values(values, skipna, fill_value=True, mask=mask)
 
     # For object type, all won't necessarily return
@@ -583,7 +612,7 @@ def nanall(
 def nansum(
     values: np.ndarray,
     *,
-    axis: int | None = None,
+    axis: AxisInt | None = None,
     skipna: bool = True,
     min_count: int = 0,
     mask: npt.NDArray[np.bool_] | None = None,
@@ -606,7 +635,7 @@ def nansum(
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([1, 2, np.nan])
     >>> nanops.nansum(s)
     3.0
@@ -628,7 +657,7 @@ def nansum(
 
 def _mask_datetimelike_result(
     result: np.ndarray | np.datetime64 | np.timedelta64,
-    axis: int | None,
+    axis: AxisInt | None,
     mask: npt.NDArray[np.bool_],
     orig_values: np.ndarray,
 ) -> np.ndarray | np.datetime64 | np.timedelta64 | NaTType:
@@ -641,7 +670,7 @@ def _mask_datetimelike_result(
         result[axis_mask] = iNaT  # type: ignore[index]
     else:
         if mask.any():
-            return NaT
+            return np.int64(iNaT).view(orig_values.dtype)
     return result
 
 
@@ -651,7 +680,7 @@ def _mask_datetimelike_result(
 def nanmean(
     values: np.ndarray,
     *,
-    axis: int | None = None,
+    axis: AxisInt | None = None,
     skipna: bool = True,
     mask: npt.NDArray[np.bool_] | None = None,
 ) -> float:
@@ -674,7 +703,7 @@ def nanmean(
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([1, 2, np.nan])
     >>> nanops.nanmean(s)
     1.5
@@ -712,7 +741,7 @@ def nanmean(
 
 
 @bottleneck_switch()
-def nanmedian(values, *, axis=None, skipna=True, mask=None):
+def nanmedian(values, *, axis: AxisInt | None = None, skipna: bool = True, mask=None):
     """
     Parameters
     ----------
@@ -730,37 +759,41 @@ def nanmedian(values, *, axis=None, skipna=True, mask=None):
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([1, np.nan, 2, 2])
     >>> nanops.nanmedian(s)
     2.0
     """
 
-    def get_median(x):
-        mask = notna(x)
-        if not skipna and not mask.all():
+    def get_median(x, _mask=None):
+        if _mask is None:
+            _mask = notna(x)
+        else:
+            _mask = ~_mask
+        if not skipna and not _mask.all():
             return np.nan
         with warnings.catch_warnings():
             # Suppress RuntimeWarning about All-NaN slice
-            warnings.filterwarnings("ignore", "All-NaN slice encountered")
-            res = np.nanmedian(x[mask])
+            warnings.filterwarnings(
+                "ignore", "All-NaN slice encountered", RuntimeWarning
+            )
+            res = np.nanmedian(x[_mask])
         return res
 
-    values, mask, dtype, _, _ = _get_values(values, skipna, mask=mask)
+    values, mask, dtype, _, _ = _get_values(values, skipna, mask=mask, fill_value=0)
     if not is_float_dtype(values.dtype):
         try:
             values = values.astype("f8")
         except ValueError as err:
             # e.g. "could not convert string to float: 'a'"
             raise TypeError(str(err)) from err
-        if mask is not None:
-            values[mask] = np.nan
+    if mask is not None:
+        values[mask] = np.nan
 
     notempty = values.size
 
     # an array from a frame
     if values.ndim > 1 and axis is not None:
-
         # there's a non-empty array to apply over otherwise numpy raises
         if notempty:
             if not skipna:
@@ -770,7 +803,9 @@ def nanmedian(values, *, axis=None, skipna=True, mask=None):
                 # fastpath for the skipna case
                 with warnings.catch_warnings():
                     # Suppress RuntimeWarning about All-NaN slice
-                    warnings.filterwarnings("ignore", "All-NaN slice encountered")
+                    warnings.filterwarnings(
+                        "ignore", "All-NaN slice encountered", RuntimeWarning
+                    )
                     res = np.nanmedian(values, axis)
 
         else:
@@ -782,13 +817,13 @@ def nanmedian(values, *, axis=None, skipna=True, mask=None):
 
     else:
         # otherwise return a scalar value
-        res = get_median(values) if notempty else np.nan
+        res = get_median(values, mask) if notempty else np.nan
     return _wrap_results(res, dtype)
 
 
 def get_empty_reduction_result(
     shape: tuple[int, ...],
-    axis: int,
+    axis: AxisInt,
     dtype: np.dtype | type[np.floating],
     fill_value: Any,
 ) -> np.ndarray:
@@ -816,10 +851,10 @@ def get_empty_reduction_result(
 def _get_counts_nanvar(
     values_shape: Shape,
     mask: npt.NDArray[np.bool_] | None,
-    axis: int | None,
+    axis: AxisInt | None,
     ddof: int,
     dtype: np.dtype = np.dtype(np.float64),
-) -> tuple[int | float | np.ndarray, int | float | np.ndarray]:
+) -> tuple[float | np.ndarray, float | np.ndarray]:
     """
     Get the count of non-null values along an axis, accounting
     for degrees of freedom.
@@ -861,7 +896,14 @@ def _get_counts_nanvar(
 
 
 @bottleneck_switch(ddof=1)
-def nanstd(values, *, axis=None, skipna=True, ddof=1, mask=None):
+def nanstd(
+    values,
+    *,
+    axis: AxisInt | None = None,
+    skipna: bool = True,
+    ddof: int = 1,
+    mask=None,
+):
     """
     Compute the standard deviation along given axis while ignoring NaNs
 
@@ -884,7 +926,7 @@ def nanstd(values, *, axis=None, skipna=True, ddof=1, mask=None):
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([1, np.nan, 2, 3])
     >>> nanops.nanstd(s)
     1.0
@@ -901,7 +943,14 @@ def nanstd(values, *, axis=None, skipna=True, ddof=1, mask=None):
 
 @disallow("M8", "m8")
 @bottleneck_switch(ddof=1)
-def nanvar(values, *, axis=None, skipna=True, ddof=1, mask=None):
+def nanvar(
+    values,
+    *,
+    axis: AxisInt | None = None,
+    skipna: bool = True,
+    ddof: int = 1,
+    mask=None,
+):
     """
     Compute the variance along given axis while ignoring NaNs
 
@@ -924,7 +973,7 @@ def nanvar(values, *, axis=None, skipna=True, ddof=1, mask=None):
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([1, np.nan, 2, 3])
     >>> nanops.nanvar(s)
     1.0
@@ -972,7 +1021,7 @@ def nanvar(values, *, axis=None, skipna=True, ddof=1, mask=None):
 def nansem(
     values: np.ndarray,
     *,
-    axis: int | None = None,
+    axis: AxisInt | None = None,
     skipna: bool = True,
     ddof: int = 1,
     mask: npt.NDArray[np.bool_] | None = None,
@@ -999,7 +1048,7 @@ def nansem(
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([1, np.nan, 2, 3])
     >>> nanops.nansem(s)
      0.5773502691896258
@@ -1012,23 +1061,25 @@ def nansem(
     if not is_float_dtype(values.dtype):
         values = values.astype("f8")
 
+    if not skipna and mask is not None and mask.any():
+        return np.nan
+
     count, _ = _get_counts_nanvar(values.shape, mask, axis, ddof, values.dtype)
-    var = nanvar(values, axis=axis, skipna=skipna, ddof=ddof)
+    var = nanvar(values, axis=axis, skipna=skipna, ddof=ddof, mask=mask)
 
     return np.sqrt(var) / np.sqrt(count)
 
 
 def _nanminmax(meth, fill_value_typ):
-    @bottleneck_switch(name="nan" + meth)
+    @bottleneck_switch(name=f"nan{meth}")
     @_datetimelike_compat
     def reduction(
         values: np.ndarray,
         *,
-        axis: int | None = None,
+        axis: AxisInt | None = None,
         skipna: bool = True,
         mask: npt.NDArray[np.bool_] | None = None,
     ) -> Dtype:
-
         values, mask, dtype, dtype_max, fill_value = _get_values(
             values, skipna, fill_value_typ=fill_value_typ, mask=mask
         )
@@ -1056,7 +1107,7 @@ nanmax = _nanminmax("max", fill_value_typ="-inf")
 def nanargmax(
     values: np.ndarray,
     *,
-    axis: int | None = None,
+    axis: AxisInt | None = None,
     skipna: bool = True,
     mask: npt.NDArray[np.bool_] | None = None,
 ) -> int | np.ndarray:
@@ -1076,7 +1127,7 @@ def nanargmax(
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> arr = np.array([1, 2, 3, np.nan, 4])
     >>> nanops.nanargmax(arr)
     4
@@ -1102,7 +1153,7 @@ def nanargmax(
 def nanargmin(
     values: np.ndarray,
     *,
-    axis: int | None = None,
+    axis: AxisInt | None = None,
     skipna: bool = True,
     mask: npt.NDArray[np.bool_] | None = None,
 ) -> int | np.ndarray:
@@ -1122,7 +1173,7 @@ def nanargmin(
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> arr = np.array([1, 2, 3, np.nan, 4])
     >>> nanops.nanargmin(arr)
     0
@@ -1149,7 +1200,7 @@ def nanargmin(
 def nanskew(
     values: np.ndarray,
     *,
-    axis: int | None = None,
+    axis: AxisInt | None = None,
     skipna: bool = True,
     mask: npt.NDArray[np.bool_] | None = None,
 ) -> float:
@@ -1176,7 +1227,7 @@ def nanskew(
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([1, np.nan, 1, 2])
     >>> nanops.nanskew(s)
     1.7320508075688787
@@ -1194,6 +1245,8 @@ def nanskew(
     if skipna and mask is not None:
         values = values.copy()
         np.putmask(values, mask, 0)
+    elif not skipna and mask is not None and mask.any():
+        return np.nan
 
     mean = values.sum(axis, dtype=np.float64) / count
     if axis is not None:
@@ -1237,7 +1290,7 @@ def nanskew(
 def nankurt(
     values: np.ndarray,
     *,
-    axis: int | None = None,
+    axis: AxisInt | None = None,
     skipna: bool = True,
     mask: npt.NDArray[np.bool_] | None = None,
 ) -> float:
@@ -1264,7 +1317,7 @@ def nankurt(
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([1, np.nan, 1, 3, 2])
     >>> nanops.nankurt(s)
     -1.2892561983471076
@@ -1282,6 +1335,8 @@ def nankurt(
     if skipna and mask is not None:
         values = values.copy()
         np.putmask(values, mask, 0)
+    elif not skipna and mask is not None and mask.any():
+        return np.nan
 
     mean = values.sum(axis, dtype=np.float64) / count
     if axis is not None:
@@ -1334,7 +1389,7 @@ def nankurt(
 def nanprod(
     values: np.ndarray,
     *,
-    axis: int | None = None,
+    axis: AxisInt | None = None,
     skipna: bool = True,
     min_count: int = 0,
     mask: npt.NDArray[np.bool_] | None = None,
@@ -1356,7 +1411,7 @@ def nanprod(
 
     Examples
     --------
-    >>> import pandas.core.nanops as nanops
+    >>> from pandas.core import nanops
     >>> s = pd.Series([1, 2, 3, np.nan])
     >>> nanops.nanprod(s)
     6.0
@@ -1376,7 +1431,7 @@ def nanprod(
 
 def _maybe_arg_null_out(
     result: np.ndarray,
-    axis: int | None,
+    axis: AxisInt | None,
     mask: npt.NDArray[np.bool_] | None,
     skipna: bool,
 ) -> np.ndarray | int:
@@ -1404,9 +1459,9 @@ def _maybe_arg_null_out(
 def _get_counts(
     values_shape: Shape,
     mask: npt.NDArray[np.bool_] | None,
-    axis: int | None,
+    axis: AxisInt | None,
     dtype: np.dtype = np.dtype(np.float64),
-) -> int | float | np.ndarray:
+) -> float | np.ndarray:
     """
     Get the count of non-null values along an axis
 
@@ -1444,7 +1499,7 @@ def _get_counts(
 
 def _maybe_null_out(
     result: np.ndarray | float | NaTType,
-    axis: int | None,
+    axis: AxisInt | None,
     mask: npt.NDArray[np.bool_] | None,
     shape: tuple[int, ...],
     min_count: int = 1,
@@ -1455,6 +1510,10 @@ def _maybe_null_out(
     Dtype
         The product of all elements on a given axis. ( NaNs are treated as 1)
     """
+    if mask is None and min_count == 0:
+        # nothing to check; short-circuit
+        return result
+
     if axis is not None and isinstance(result, np.ndarray):
         if mask is not None:
             null_mask = (mask.shape[axis] - mask.sum(axis) - min_count) < 0
@@ -1468,15 +1527,20 @@ def _maybe_null_out(
             if is_numeric_dtype(result):
                 if np.iscomplexobj(result):
                     result = result.astype("c16")
-                else:
-                    result = result.astype("f8")
+                elif not is_float_dtype(result):
+                    result = result.astype("f8", copy=False)
                 result[null_mask] = np.nan
             else:
                 # GH12941, use None to auto cast null
                 result[null_mask] = None
     elif result is not NaT:
         if check_below_min_count(shape, mask, min_count):
-            result = np.nan
+            result_dtype = getattr(result, "dtype", None)
+            if is_float_dtype(result_dtype):
+                # error: Item "None" of "Optional[Any]" has no attribute "type"
+                result = result_dtype.type("nan")  # type: ignore[union-attr]
+            else:
+                result = np.nan
 
     return result
 
@@ -1523,8 +1587,12 @@ def _zero_out_fperr(arg):
 
 @disallow("M8", "m8")
 def nancorr(
-    a: np.ndarray, b: np.ndarray, *, method="pearson", min_periods: int | None = None
-):
+    a: np.ndarray,
+    b: np.ndarray,
+    *,
+    method: CorrelationMethod = "pearson",
+    min_periods: int | None = None,
+) -> float:
     """
     a, b: ndarrays
     """
@@ -1546,7 +1614,9 @@ def nancorr(
     return f(a, b)
 
 
-def get_corr_func(method):
+def get_corr_func(
+    method: CorrelationMethod,
+) -> Callable[[np.ndarray, np.ndarray], float]:
     if method == "kendall":
         from scipy.stats import kendalltau
 
@@ -1583,7 +1653,7 @@ def nancov(
     *,
     min_periods: int | None = None,
     ddof: int | None = 1,
-):
+) -> float:
     if len(a) != len(b):
         raise AssertionError("Operands to nancov must have same size")
 
@@ -1681,53 +1751,11 @@ def na_accum_func(values: ArrayLike, accum_func, *, skipna: bool) -> ArrayLike:
         np.minimum.accumulate: (np.inf, np.nan),
     }[accum_func]
 
+    # This should go through ea interface
+    assert values.dtype.kind not in ["m", "M"]
+
     # We will be applying this function to block values
-    if values.dtype.kind in ["m", "M"]:
-        # GH#30460, GH#29058
-        # numpy 1.18 started sorting NaTs at the end instead of beginning,
-        #  so we need to work around to maintain backwards-consistency.
-        orig_dtype = values.dtype
-
-        # We need to define mask before masking NaTs
-        mask = isna(values)
-
-        y = values.view("i8")
-        # Note: the accum_func comparison fails as an "is" comparison
-        changed = accum_func == np.minimum.accumulate
-
-        try:
-            if changed:
-                y[mask] = lib.i8max
-
-            result = accum_func(y, axis=0)
-        finally:
-            if changed:
-                # restore NaT elements
-                y[mask] = iNaT
-
-        if skipna:
-            result[mask] = iNaT
-        elif accum_func == np.minimum.accumulate:
-            # Restore NaTs that we masked previously
-            nz = (~np.asarray(mask)).nonzero()[0]
-            if len(nz):
-                # everything up to the first non-na entry stays NaT
-                result[: nz[0]] = iNaT
-
-        if isinstance(values.dtype, np.dtype):
-            result = result.view(orig_dtype)
-        else:
-            # DatetimeArray/TimedeltaArray
-            # TODO: have this case go through a DTA method?
-            # For DatetimeTZDtype, view result as M8[ns]
-            npdtype = orig_dtype if isinstance(orig_dtype, np.dtype) else "M8[ns]"
-            # Item "type" of "Union[Type[ExtensionArray], Type[ndarray[Any, Any]]]"
-            # has no attribute "_simple_new"
-            result = type(values)._simple_new(  # type: ignore[union-attr]
-                result.view(npdtype), dtype=orig_dtype
-            )
-
-    elif skipna and not issubclass(values.dtype.type, (np.integer, np.bool_)):
+    if skipna and not issubclass(values.dtype.type, (np.integer, np.bool_)):
         vals = values.copy()
         mask = isna(vals)
         vals[mask] = mask_a
