@@ -1,4 +1,5 @@
 import calendar
+from datetime import datetime
 import locale
 import unicodedata
 
@@ -6,144 +7,37 @@ import numpy as np
 import pytest
 
 import pandas as pd
-from pandas import DatetimeIndex, Index, Timestamp, date_range, datetime, offsets
-import pandas.util.testing as tm
+from pandas import (
+    DatetimeIndex,
+    Index,
+    Timedelta,
+    Timestamp,
+    date_range,
+    offsets,
+)
+import pandas._testing as tm
+from pandas.core.arrays import DatetimeArray
 
-
-class TestTimeSeries:
-    def test_pass_datetimeindex_to_index(self):
-        # Bugs in #1396
-        rng = date_range("1/1/2000", "3/1/2000")
-        idx = Index(rng, dtype=object)
-
-        expected = Index(rng.to_pydatetime(), dtype=object)
-
-        tm.assert_numpy_array_equal(idx.values, expected.values)
-
-    def test_range_edges(self):
-        # GH#13672
-        idx = pd.date_range(
-            start=Timestamp("1970-01-01 00:00:00.000000001"),
-            end=Timestamp("1970-01-01 00:00:00.000000004"),
-            freq="N",
-        )
-        exp = DatetimeIndex(
-            [
-                "1970-01-01 00:00:00.000000001",
-                "1970-01-01 00:00:00.000000002",
-                "1970-01-01 00:00:00.000000003",
-                "1970-01-01 00:00:00.000000004",
-            ]
-        )
-        tm.assert_index_equal(idx, exp)
-
-        idx = pd.date_range(
-            start=Timestamp("1970-01-01 00:00:00.000000004"),
-            end=Timestamp("1970-01-01 00:00:00.000000001"),
-            freq="N",
-        )
-        exp = DatetimeIndex([])
-        tm.assert_index_equal(idx, exp)
-
-        idx = pd.date_range(
-            start=Timestamp("1970-01-01 00:00:00.000000001"),
-            end=Timestamp("1970-01-01 00:00:00.000000001"),
-            freq="N",
-        )
-        exp = DatetimeIndex(["1970-01-01 00:00:00.000000001"])
-        tm.assert_index_equal(idx, exp)
-
-        idx = pd.date_range(
-            start=Timestamp("1970-01-01 00:00:00.000001"),
-            end=Timestamp("1970-01-01 00:00:00.000004"),
-            freq="U",
-        )
-        exp = DatetimeIndex(
-            [
-                "1970-01-01 00:00:00.000001",
-                "1970-01-01 00:00:00.000002",
-                "1970-01-01 00:00:00.000003",
-                "1970-01-01 00:00:00.000004",
-            ]
-        )
-        tm.assert_index_equal(idx, exp)
-
-        idx = pd.date_range(
-            start=Timestamp("1970-01-01 00:00:00.001"),
-            end=Timestamp("1970-01-01 00:00:00.004"),
-            freq="L",
-        )
-        exp = DatetimeIndex(
-            [
-                "1970-01-01 00:00:00.001",
-                "1970-01-01 00:00:00.002",
-                "1970-01-01 00:00:00.003",
-                "1970-01-01 00:00:00.004",
-            ]
-        )
-        tm.assert_index_equal(idx, exp)
-
-        idx = pd.date_range(
-            start=Timestamp("1970-01-01 00:00:01"),
-            end=Timestamp("1970-01-01 00:00:04"),
-            freq="S",
-        )
-        exp = DatetimeIndex(
-            [
-                "1970-01-01 00:00:01",
-                "1970-01-01 00:00:02",
-                "1970-01-01 00:00:03",
-                "1970-01-01 00:00:04",
-            ]
-        )
-        tm.assert_index_equal(idx, exp)
-
-        idx = pd.date_range(
-            start=Timestamp("1970-01-01 00:01"),
-            end=Timestamp("1970-01-01 00:04"),
-            freq="T",
-        )
-        exp = DatetimeIndex(
-            [
-                "1970-01-01 00:01",
-                "1970-01-01 00:02",
-                "1970-01-01 00:03",
-                "1970-01-01 00:04",
-            ]
-        )
-        tm.assert_index_equal(idx, exp)
-
-        idx = pd.date_range(
-            start=Timestamp("1970-01-01 01:00"),
-            end=Timestamp("1970-01-01 04:00"),
-            freq="H",
-        )
-        exp = DatetimeIndex(
-            [
-                "1970-01-01 01:00",
-                "1970-01-01 02:00",
-                "1970-01-01 03:00",
-                "1970-01-01 04:00",
-            ]
-        )
-        tm.assert_index_equal(idx, exp)
-
-        idx = pd.date_range(
-            start=Timestamp("1970-01-01"), end=Timestamp("1970-01-04"), freq="D"
-        )
-        exp = DatetimeIndex(["1970-01-01", "1970-01-02", "1970-01-03", "1970-01-04"])
-        tm.assert_index_equal(idx, exp)
+from pandas.tseries.frequencies import to_offset
 
 
 class TestDatetime64:
+    def test_no_millisecond_field(self):
+        msg = "type object 'DatetimeIndex' has no attribute 'millisecond'"
+        with pytest.raises(AttributeError, match=msg):
+            DatetimeIndex.millisecond
+
+        msg = "'DatetimeIndex' object has no attribute 'millisecond'"
+        with pytest.raises(AttributeError, match=msg):
+            DatetimeIndex([]).millisecond
+
     def test_datetimeindex_accessors(self):
-        dti_naive = pd.date_range(freq="D", start=datetime(1998, 1, 1), periods=365)
+        dti_naive = date_range(freq="D", start=datetime(1998, 1, 1), periods=365)
         # GH#13303
-        dti_tz = pd.date_range(
+        dti_tz = date_range(
             freq="D", start=datetime(1998, 1, 1), periods=365, tz="US/Eastern"
         )
         for dti in [dti_naive, dti_tz]:
-
             assert dti.year[0] == 1998
             assert dti.month[0] == 1
             assert dti.day[0] == 1
@@ -156,8 +50,8 @@ class TestDatetime64:
             assert dti.dayofyear[0] == 1
             assert dti.dayofyear[120] == 121
 
-            assert dti.weekofyear[0] == 1
-            assert dti.weekofyear[120] == 18
+            assert dti.isocalendar().week[0] == 1
+            assert dti.isocalendar().week[120] == 18
 
             assert dti.quarter[0] == 1
             assert dti.quarter[120] == 2
@@ -192,7 +86,7 @@ class TestDatetime64:
             assert len(dti.microsecond) == 365
             assert len(dti.dayofweek) == 365
             assert len(dti.dayofyear) == 365
-            assert len(dti.weekofyear) == 365
+            assert len(dti.isocalendar()) == 365
             assert len(dti.quarter) == 365
             assert len(dti.is_month_start) == 365
             assert len(dti.is_month_end) == 365
@@ -200,19 +94,18 @@ class TestDatetime64:
             assert len(dti.is_quarter_end) == 365
             assert len(dti.is_year_start) == 365
             assert len(dti.is_year_end) == 365
-            assert len(dti.weekday_name) == 365
 
             dti.name = "name"
 
             # non boolean accessors -> return Index
-            for accessor in DatetimeIndex._field_ops:
+            for accessor in DatetimeArray._field_ops:
                 res = getattr(dti, accessor)
                 assert len(res) == 365
                 assert isinstance(res, Index)
                 assert res.name == "name"
 
             # boolean accessors -> return array
-            for accessor in DatetimeIndex._bool_ops:
+            for accessor in DatetimeArray._bool_ops:
                 res = getattr(dti, accessor)
                 assert len(res) == 365
                 assert isinstance(res, np.ndarray)
@@ -225,13 +118,15 @@ class TestDatetime64:
             exp = DatetimeIndex([], freq="D", tz=dti.tz, name="name")
             tm.assert_index_equal(res, exp)
 
-        dti = pd.date_range(freq="BQ-FEB", start=datetime(1998, 1, 1), periods=4)
+    def test_datetimeindex_accessors2(self):
+        dti = date_range(freq="BQ-FEB", start=datetime(1998, 1, 1), periods=4)
 
         assert sum(dti.is_quarter_start) == 0
         assert sum(dti.is_quarter_end) == 4
         assert sum(dti.is_year_start) == 0
         assert sum(dti.is_year_end) == 1
 
+    def test_datetimeindex_accessors3(self):
         # Ensure is_start/end accessors throw ValueError for CustomBusinessDay,
         bday_egypt = offsets.CustomBusinessDay(weekmask="Sun Mon Tue Wed Thu")
         dti = date_range(datetime(2013, 4, 30), periods=5, freq=bday_egypt)
@@ -239,41 +134,51 @@ class TestDatetime64:
         with pytest.raises(ValueError, match=msg):
             dti.is_month_start
 
+    def test_datetimeindex_accessors4(self):
         dti = DatetimeIndex(["2000-01-01", "2000-01-02", "2000-01-03"])
 
         assert dti.is_month_start[0] == 1
 
+    def test_datetimeindex_accessors5(self):
+        freq_m = to_offset("M")
+        bm = to_offset("BM")
+        qfeb = to_offset("Q-FEB")
+        qsfeb = to_offset("QS-FEB")
+        bq = to_offset("BQ")
+        bqs_apr = to_offset("BQS-APR")
+        as_nov = to_offset("AS-NOV")
+
         tests = [
-            (Timestamp("2013-06-01", freq="M").is_month_start, 1),
-            (Timestamp("2013-06-01", freq="BM").is_month_start, 0),
-            (Timestamp("2013-06-03", freq="M").is_month_start, 0),
-            (Timestamp("2013-06-03", freq="BM").is_month_start, 1),
-            (Timestamp("2013-02-28", freq="Q-FEB").is_month_end, 1),
-            (Timestamp("2013-02-28", freq="Q-FEB").is_quarter_end, 1),
-            (Timestamp("2013-02-28", freq="Q-FEB").is_year_end, 1),
-            (Timestamp("2013-03-01", freq="Q-FEB").is_month_start, 1),
-            (Timestamp("2013-03-01", freq="Q-FEB").is_quarter_start, 1),
-            (Timestamp("2013-03-01", freq="Q-FEB").is_year_start, 1),
-            (Timestamp("2013-03-31", freq="QS-FEB").is_month_end, 1),
-            (Timestamp("2013-03-31", freq="QS-FEB").is_quarter_end, 0),
-            (Timestamp("2013-03-31", freq="QS-FEB").is_year_end, 0),
-            (Timestamp("2013-02-01", freq="QS-FEB").is_month_start, 1),
-            (Timestamp("2013-02-01", freq="QS-FEB").is_quarter_start, 1),
-            (Timestamp("2013-02-01", freq="QS-FEB").is_year_start, 1),
-            (Timestamp("2013-06-30", freq="BQ").is_month_end, 0),
-            (Timestamp("2013-06-30", freq="BQ").is_quarter_end, 0),
-            (Timestamp("2013-06-30", freq="BQ").is_year_end, 0),
-            (Timestamp("2013-06-28", freq="BQ").is_month_end, 1),
-            (Timestamp("2013-06-28", freq="BQ").is_quarter_end, 1),
-            (Timestamp("2013-06-28", freq="BQ").is_year_end, 0),
-            (Timestamp("2013-06-30", freq="BQS-APR").is_month_end, 0),
-            (Timestamp("2013-06-30", freq="BQS-APR").is_quarter_end, 0),
-            (Timestamp("2013-06-30", freq="BQS-APR").is_year_end, 0),
-            (Timestamp("2013-06-28", freq="BQS-APR").is_month_end, 1),
-            (Timestamp("2013-06-28", freq="BQS-APR").is_quarter_end, 1),
-            (Timestamp("2013-03-29", freq="BQS-APR").is_year_end, 1),
-            (Timestamp("2013-11-01", freq="AS-NOV").is_year_start, 1),
-            (Timestamp("2013-10-31", freq="AS-NOV").is_year_end, 1),
+            (freq_m.is_month_start(Timestamp("2013-06-01")), 1),
+            (bm.is_month_start(Timestamp("2013-06-01")), 0),
+            (freq_m.is_month_start(Timestamp("2013-06-03")), 0),
+            (bm.is_month_start(Timestamp("2013-06-03")), 1),
+            (qfeb.is_month_end(Timestamp("2013-02-28")), 1),
+            (qfeb.is_quarter_end(Timestamp("2013-02-28")), 1),
+            (qfeb.is_year_end(Timestamp("2013-02-28")), 1),
+            (qfeb.is_month_start(Timestamp("2013-03-01")), 1),
+            (qfeb.is_quarter_start(Timestamp("2013-03-01")), 1),
+            (qfeb.is_year_start(Timestamp("2013-03-01")), 1),
+            (qsfeb.is_month_end(Timestamp("2013-03-31")), 1),
+            (qsfeb.is_quarter_end(Timestamp("2013-03-31")), 0),
+            (qsfeb.is_year_end(Timestamp("2013-03-31")), 0),
+            (qsfeb.is_month_start(Timestamp("2013-02-01")), 1),
+            (qsfeb.is_quarter_start(Timestamp("2013-02-01")), 1),
+            (qsfeb.is_year_start(Timestamp("2013-02-01")), 1),
+            (bq.is_month_end(Timestamp("2013-06-30")), 0),
+            (bq.is_quarter_end(Timestamp("2013-06-30")), 0),
+            (bq.is_year_end(Timestamp("2013-06-30")), 0),
+            (bq.is_month_end(Timestamp("2013-06-28")), 1),
+            (bq.is_quarter_end(Timestamp("2013-06-28")), 1),
+            (bq.is_year_end(Timestamp("2013-06-28")), 0),
+            (bqs_apr.is_month_end(Timestamp("2013-06-30")), 0),
+            (bqs_apr.is_quarter_end(Timestamp("2013-06-30")), 0),
+            (bqs_apr.is_year_end(Timestamp("2013-06-30")), 0),
+            (bqs_apr.is_month_end(Timestamp("2013-06-28")), 1),
+            (bqs_apr.is_quarter_end(Timestamp("2013-06-28")), 1),
+            (bqs_apr.is_year_end(Timestamp("2013-03-29")), 1),
+            (as_nov.is_year_start(Timestamp("2013-11-01")), 1),
+            (as_nov.is_year_end(Timestamp("2013-10-31")), 1),
             (Timestamp("2012-02-01").days_in_month, 29),
             (Timestamp("2013-02-01").days_in_month, 28),
         ]
@@ -281,17 +186,19 @@ class TestDatetime64:
         for ts, value in tests:
             assert ts == value
 
+    def test_datetimeindex_accessors6(self):
         # GH 6538: Check that DatetimeIndex and its TimeStamp elements
         # return the same weekofyear accessor close to new year w/ tz
         dates = ["2013/12/29", "2013/12/30", "2013/12/31"]
         dates = DatetimeIndex(dates, tz="Europe/Brussels")
         expected = [52, 1, 1]
-        assert dates.weekofyear.tolist() == expected
+        assert dates.isocalendar().week.tolist() == expected
         assert [d.weekofyear for d in dates] == expected
 
     # GH 12806
+    # error: Unsupported operand types for + ("List[None]" and "List[str]")
     @pytest.mark.parametrize(
-        "time_locale", [None] if tm.get_locales() is None else [None] + tm.get_locales()
+        "time_locale", [None] + tm.get_locales()  # type: ignore[operator]
     )
     def test_datetime_name_accessors(self, time_locale):
         # Test Monday -> Sunday and January -> December, in that sequence
@@ -327,7 +234,7 @@ class TestDatetime64:
                 expected_months = calendar.month_name[1:]
 
         # GH#11128
-        dti = pd.date_range(freq="D", start=datetime(1998, 1, 1), periods=365)
+        dti = date_range(freq="D", start=datetime(1998, 1, 1), periods=365)
         english_days = [
             "Monday",
             "Tuesday",
@@ -339,11 +246,9 @@ class TestDatetime64:
         ]
         for day, name, eng_name in zip(range(4, 11), expected_days, english_days):
             name = name.capitalize()
-            assert dti.weekday_name[day] == eng_name
             assert dti.day_name(locale=time_locale)[day] == name
+            assert dti.day_name(locale=None)[day] == eng_name
             ts = Timestamp(datetime(2016, 4, day))
-            with tm.assert_produces_warning(FutureWarning, check_stacklevel=False):
-                assert ts.weekday_name == eng_name
             assert ts.day_name(locale=time_locale) == name
         dti = dti.append(DatetimeIndex([pd.NaT]))
         assert np.isnan(dti.day_name(locale=time_locale)[-1])
@@ -351,7 +256,7 @@ class TestDatetime64:
         assert np.isnan(ts.day_name(locale=time_locale))
 
         # GH#12805
-        dti = pd.date_range(freq="M", start="2012", end="2013")
+        dti = date_range(freq="M", start="2012", end="2013")
         result = dti.month_name(locale=time_locale)
         expected = Index([month.capitalize() for month in expected_months])
 
@@ -375,8 +280,9 @@ class TestDatetime64:
 
     def test_nanosecond_field(self):
         dti = DatetimeIndex(np.arange(10))
+        expected = Index(np.arange(10, dtype=np.int32))
 
-        tm.assert_index_equal(dti.nanosecond, pd.Index(np.arange(10, dtype=np.int64)))
+        tm.assert_index_equal(dti.nanosecond, expected)
 
 
 def test_iter_readonly():
@@ -385,3 +291,15 @@ def test_iter_readonly():
     arr.setflags(write=False)
     dti = pd.to_datetime(arr)
     list(dti)
+
+
+def test_add_timedelta_preserves_freq():
+    # GH#37295 should hold for any DTI with freq=None or Tick freq
+    tz = "Canada/Eastern"
+    dti = date_range(
+        start=Timestamp("2019-03-26 00:00:00-0400", tz=tz),
+        end=Timestamp("2020-10-17 00:00:00-0400", tz=tz),
+        freq="D",
+    )
+    result = dti + Timedelta(days=1)
+    assert result.freq == dti.freq
