@@ -17,46 +17,44 @@ from pandas import (
     Index,
 )
 import pandas._testing as tm
+from pandas.tests.io.test_compression import _compression_to_extension
 
-import pandas.io.common as icom
 from pandas.io.common import get_handle
 from pandas.io.xml import read_xml
 
-"""
-CHECKLIST
+# CHECKLIST
 
-[x] - ValueError: "Values for parser can only be lxml or etree."
+# [x] - ValueError: "Values for parser can only be lxml or etree."
 
-etree
-[x] - ImportError: "lxml not found, please install or use the etree parser."
-[X] - TypeError: "...is not a valid type for attr_cols"
-[X] - TypeError: "...is not a valid type for elem_cols"
-[X] - LookupError: "unknown encoding"
-[X] - KeyError: "...is not included in namespaces"
-[X] - KeyError: "no valid column"
-[X] - ValueError: "To use stylesheet, you need lxml installed..."
-[]  - OSError: (NEED PERMISSOIN ISSUE, DISK FULL, ETC.)
-[X] - FileNotFoundError: "No such file or directory"
-[X] - PermissionError: "Forbidden"
+# etree
+# [x] - ImportError: "lxml not found, please install or use the etree parser."
+# [X] - TypeError: "...is not a valid type for attr_cols"
+# [X] - TypeError: "...is not a valid type for elem_cols"
+# [X] - LookupError: "unknown encoding"
+# [X] - KeyError: "...is not included in namespaces"
+# [X] - KeyError: "no valid column"
+# [X] - ValueError: "To use stylesheet, you need lxml installed..."
+# []  - OSError: (NEED PERMISSOIN ISSUE, DISK FULL, ETC.)
+# [X] - FileNotFoundError: "No such file or directory"
+# [X] - PermissionError: "Forbidden"
 
-lxml
-[X] - TypeError: "...is not a valid type for attr_cols"
-[X] - TypeError: "...is not a valid type for elem_cols"
-[X] - LookupError: "unknown encoding"
-[]  - OSError: (NEED PERMISSOIN ISSUE, DISK FULL, ETC.)
-[X] - FileNotFoundError: "No such file or directory"
-[X] - KeyError: "...is not included in namespaces"
-[X] - KeyError: "no valid column"
-[X] - ValueError: "stylesheet is not a url, file, or xml string."
-[]  - LookupError: (NEED WRONG ENCODING FOR FILE OUTPUT)
-[]  - URLError: (USUALLY DUE TO NETWORKING)
-[]  - HTTPError: (NEED AN ONLINE STYLESHEET)
-[X] - OSError: "failed to load external entity"
-[X] - XMLSyntaxError: "Opening and ending tag mismatch"
-[X] - XSLTApplyError: "Cannot resolve URI"
-[X] - XSLTParseError: "failed to compile"
-[X] - PermissionError: "Forbidden"
-"""
+# lxml
+# [X] - TypeError: "...is not a valid type for attr_cols"
+# [X] - TypeError: "...is not a valid type for elem_cols"
+# [X] - LookupError: "unknown encoding"
+# []  - OSError: (NEED PERMISSOIN ISSUE, DISK FULL, ETC.)
+# [X] - FileNotFoundError: "No such file or directory"
+# [X] - KeyError: "...is not included in namespaces"
+# [X] - KeyError: "no valid column"
+# [X] - ValueError: "stylesheet is not a url, file, or xml string."
+# []  - LookupError: (NEED WRONG ENCODING FOR FILE OUTPUT)
+# []  - URLError: (USUALLY DUE TO NETWORKING)
+# []  - HTTPError: (NEED AN ONLINE STYLESHEET)
+# [X] - OSError: "failed to load external entity"
+# [X] - XMLSyntaxError: "Opening and ending tag mismatch"
+# [X] - XSLTApplyError: "Cannot resolve URI"
+# [X] - XSLTParseError: "failed to compile"
+# [X] - PermissionError: "Forbidden"
 
 geom_df = DataFrame(
     {
@@ -363,21 +361,21 @@ na_expected = """\
 </data>"""
 
 
-def test_na_elem_output(datapath, parser):
+def test_na_elem_output(parser):
     output = geom_df.to_xml(parser=parser)
     output = equalize_decl(output)
 
     assert output == na_expected
 
 
-def test_na_empty_str_elem_option(datapath, parser):
+def test_na_empty_str_elem_option(parser):
     output = geom_df.to_xml(na_rep="", parser=parser)
     output = equalize_decl(output)
 
     assert output == na_expected
 
 
-def test_na_empty_elem_option(datapath, parser):
+def test_na_empty_elem_option(parser):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -410,7 +408,7 @@ def test_na_empty_elem_option(datapath, parser):
 # ATTR_COLS
 
 
-def test_attrs_cols_nan_output(datapath, parser):
+def test_attrs_cols_nan_output(parser):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -425,7 +423,7 @@ def test_attrs_cols_nan_output(datapath, parser):
     assert output == expected
 
 
-def test_attrs_cols_prefix(datapath, parser):
+def test_attrs_cols_prefix(parser):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <doc:data xmlns:doc="http://example.xom">
@@ -461,7 +459,7 @@ def test_attrs_wrong_type(parser):
 # ELEM_COLS
 
 
-def test_elems_cols_nan_output(datapath, parser):
+def test_elems_cols_nan_output(parser):
     elems_cols_expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -500,7 +498,7 @@ def test_elems_wrong_type(parser):
         geom_df.to_xml(elem_cols='"shape", "degree", "sides"', parser=parser)
 
 
-def test_elems_and_attrs_cols(datapath, parser):
+def test_elems_and_attrs_cols(parser):
     elems_cols_expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -532,7 +530,7 @@ def test_elems_and_attrs_cols(datapath, parser):
 # HIERARCHICAL COLUMNS
 
 
-def test_hierarchical_columns(datapath, parser):
+def test_hierarchical_columns(parser):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -579,7 +577,7 @@ def test_hierarchical_columns(datapath, parser):
     assert output == expected
 
 
-def test_hierarchical_attrs_columns(datapath, parser):
+def test_hierarchical_attrs_columns(parser):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -609,7 +607,7 @@ sum_mass="2667.54" mean_mass="333.44"/>
 # MULTIINDEX
 
 
-def test_multi_index(datapath, parser):
+def test_multi_index(parser):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -648,7 +646,7 @@ def test_multi_index(datapath, parser):
     assert output == expected
 
 
-def test_multi_index_attrs_cols(datapath, parser):
+def test_multi_index_attrs_cols(parser):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -741,7 +739,6 @@ def test_namespace_prefix(parser):
 
 def test_missing_prefix_in_nmsp(parser):
     with pytest.raises(KeyError, match=("doc is not included in namespaces")):
-
         geom_df.to_xml(
             namespaces={"": "http://example.com"}, prefix="doc", parser=parser
         )
@@ -1020,7 +1017,7 @@ def test_stylesheet_buffered_reader(datapath, mode):
 
 
 @td.skip_if_no("lxml")
-def test_stylesheet_wrong_path(datapath):
+def test_stylesheet_wrong_path():
     from lxml.etree import XMLSyntaxError
 
     xsl = os.path.join("data", "xml", "row_field_output.xslt")
@@ -1037,9 +1034,16 @@ def test_stylesheet_wrong_path(datapath):
 def test_empty_string_stylesheet(val):
     from lxml.etree import XMLSyntaxError
 
-    with pytest.raises(
-        XMLSyntaxError, match=("Document is empty|Start tag expected, '<' not found")
-    ):
+    msg = "|".join(
+        [
+            "Document is empty",
+            "Start tag expected, '<' not found",
+            # Seen on Mac with lxml 4.9.1
+            r"None \(line 0\)",
+        ]
+    )
+
+    with pytest.raises(XMLSyntaxError, match=msg):
         geom_df.to_xml(stylesheet=val)
 
 
@@ -1102,7 +1106,7 @@ def test_incorrect_xsl_eval():
 
 
 @td.skip_if_no("lxml")
-def test_incorrect_xsl_apply(parser):
+def test_incorrect_xsl_apply():
     from lxml.etree import XSLTApplyError
 
     xsl = """\
@@ -1122,7 +1126,7 @@ def test_incorrect_xsl_apply(parser):
             geom_df.to_xml(path, stylesheet=xsl)
 
 
-def test_stylesheet_with_etree(datapath):
+def test_stylesheet_with_etree():
     xsl = """\
 <xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
     <xsl:output method="xml" encoding="utf-8" indent="yes" />
@@ -1160,7 +1164,7 @@ def test_style_to_csv():
     </xsl:template>
 </xsl:stylesheet>"""
 
-    out_csv = geom_df.to_csv(line_terminator="\n")
+    out_csv = geom_df.to_csv(lineterminator="\n")
 
     if out_csv is not None:
         out_csv = out_csv.strip()
@@ -1292,7 +1296,7 @@ def test_compression_output(parser, compression_only):
 
 
 def test_filename_and_suffix_comp(parser, compression_only):
-    compfile = "xml." + icom._compression_to_extension[compression_only]
+    compfile = "xml." + _compression_to_extension[compression_only]
     with tm.ensure_clean(filename=compfile) as path:
         geom_df.to_xml(path, parser=parser, compression=compression_only)
 
@@ -1322,7 +1326,7 @@ def test_ea_dtypes(any_numeric_ea_dtype, parser):
     assert equalize_decl(result).strip() == expected
 
 
-def test_unsuported_compression(datapath, parser):
+def test_unsuported_compression(parser):
     with pytest.raises(ValueError, match="Unrecognized compression type"):
         with tm.ensure_clean() as path:
             geom_df.to_xml(path, parser=parser, compression="7z")
