@@ -1,15 +1,33 @@
 import contextlib
+from typing import Generator
 
 import pytest
 
 import pandas as pd
 import pandas._testing as tm
+from pandas.core import accessor
+
+
+def test_dirname_mixin():
+    # GH37173
+
+    class X(accessor.DirNamesMixin):
+        x = 1
+        y: int
+
+        def __init__(self) -> None:
+            self.z = 3
+
+    result = [attr_name for attr_name in dir(X()) if not attr_name.startswith("_")]
+
+    assert result == ["x", "z"]
 
 
 @contextlib.contextmanager
-def ensure_removed(obj, attr):
+def ensure_removed(obj, attr) -> Generator[None, None, None]:
     """Ensure that an attribute added to 'obj' during the test is
-    removed when we're done"""
+    removed when we're done
+    """
     try:
         yield
     finally:
@@ -21,7 +39,7 @@ def ensure_removed(obj, attr):
 
 
 class MyAccessor:
-    def __init__(self, obj):
+    def __init__(self, obj) -> None:
         self.obj = obj
         self.item = "item"
 
@@ -80,12 +98,11 @@ def test_overwrite_warns():
 
 
 def test_raises_attribute_error():
-
     with ensure_removed(pd.Series, "bad"):
 
         @pd.api.extensions.register_series_accessor("bad")
         class Bad:
-            def __init__(self, data):
+            def __init__(self, data) -> None:
                 raise AttributeError("whoops")
 
         with pytest.raises(AttributeError, match="whoops"):
