@@ -1,12 +1,13 @@
 from datetime import (
     date,
     datetime,
+    timedelta,
+    timezone,
 )
 
 from dateutil.tz.tz import tzoffset
 import numpy as np
 import pytest
-import pytz
 
 from pandas._libs import (
     iNaT,
@@ -63,18 +64,20 @@ def test_parsing_timezone_offsets(dt_string, expected_tz):
     result, result_tz = tslib.array_to_datetime(arr)
 
     tm.assert_numpy_array_equal(result, expected)
-    assert result_tz is pytz.FixedOffset(expected_tz)
+    assert result_tz == timezone(timedelta(minutes=expected_tz))
 
 
 def test_parsing_non_iso_timezone_offset():
     dt_string = "01-01-2013T00:00:00.000000000+0000"
     arr = np.array([dt_string], dtype=object)
 
-    result, result_tz = tslib.array_to_datetime(arr)
+    with tm.assert_produces_warning(None):
+        # GH#50949 should not get tzlocal-deprecation warning here
+        result, result_tz = tslib.array_to_datetime(arr)
     expected = np.array([np.datetime64("2013-01-01 00:00:00.000000000")])
 
     tm.assert_numpy_array_equal(result, expected)
-    assert result_tz is pytz.FixedOffset(0)
+    assert result_tz is timezone.utc
 
 
 def test_parsing_different_timezone_offsets():
@@ -125,7 +128,7 @@ def test_coerce_outside_ns_bounds(invalid_date, errors):
     kwargs = {"values": arr, "errors": errors}
 
     if errors == "raise":
-        msg = "Out of bounds nanosecond timestamp"
+        msg = "^Out of bounds nanosecond timestamp: .*, at position 0$"
 
         with pytest.raises(ValueError, match=msg):
             tslib.array_to_datetime(**kwargs)
@@ -170,7 +173,7 @@ def test_to_datetime_barely_out_of_bounds():
     # Close enough to bounds that dropping nanos
     # would result in an in-bounds datetime.
     arr = np.array(["2262-04-11 23:47:16.854775808"], dtype=object)
-    msg = "Out of bounds nanosecond timestamp: 2262-04-11 23:47:16"
+    msg = "^Out of bounds nanosecond timestamp: 2262-04-11 23:47:16, at position 0$"
 
     with pytest.raises(tslib.OutOfBoundsDatetime, match=msg):
         tslib.array_to_datetime(arr)
