@@ -1,3 +1,7 @@
+from __future__ import annotations
+
+from typing import Iterable
+
 import numpy as np
 
 import pandas as pd
@@ -9,34 +13,37 @@ class TablePlotter:
     Used in merging.rst
     """
 
-    def __init__(self, cell_width=0.37, cell_height=0.25, font_size=7.5):
+    def __init__(
+        self,
+        cell_width: float = 0.37,
+        cell_height: float = 0.25,
+        font_size: float = 7.5,
+    ) -> None:
         self.cell_width = cell_width
         self.cell_height = cell_height
         self.font_size = font_size
 
-    def _shape(self, df):
+    def _shape(self, df: pd.DataFrame) -> tuple[int, int]:
         """
-        Calculate table chape considering index levels.
+        Calculate table shape considering index levels.
         """
-
         row, col = df.shape
         return row + df.columns.nlevels, col + df.index.nlevels
 
-    def _get_cells(self, left, right, vertical):
+    def _get_cells(self, left, right, vertical) -> tuple[int, int]:
         """
         Calculate appropriate figure size based on left and right data.
         """
-
         if vertical:
             # calculate required number of cells
-            vcells = max(sum(self._shape(l)[0] for l in left), self._shape(right)[0])
-            hcells = max(self._shape(l)[1] for l in left) + self._shape(right)[1]
+            vcells = max(sum(self._shape(df)[0] for df in left), self._shape(right)[0])
+            hcells = max(self._shape(df)[1] for df in left) + self._shape(right)[1]
         else:
-            vcells = max([self._shape(l)[0] for l in left] + [self._shape(right)[0]])
-            hcells = sum([self._shape(l)[1] for l in left] + [self._shape(right)[1]])
+            vcells = max([self._shape(df)[0] for df in left] + [self._shape(right)[0]])
+            hcells = sum([self._shape(df)[1] for df in left] + [self._shape(right)[1]])
         return hcells, vcells
 
-    def plot(self, left, right, labels=None, vertical=True):
+    def plot(self, left, right, labels: Iterable[str] = (), vertical: bool = True):
         """
         Plot left / right DataFrames in specified layout.
 
@@ -45,15 +52,15 @@ class TablePlotter:
         left : list of DataFrames before operation is applied
         right : DataFrame of operation result
         labels : list of str to be drawn as titles of left DataFrames
-        vertical : bool
+        vertical : bool, default True
             If True, use vertical layout. If False, use horizontal layout.
         """
+        from matplotlib import gridspec
         import matplotlib.pyplot as plt
-        import matplotlib.gridspec as gridspec
 
         if not isinstance(left, list):
             left = [left]
-        left = [self._conv(l) for l in left]
+        left = [self._conv(df) for df in left]
         right = self._conv(right)
 
         hcells, vcells = self._get_cells(left, right, vertical)
@@ -68,11 +75,11 @@ class TablePlotter:
         if vertical:
             gs = gridspec.GridSpec(len(left), hcells)
             # left
-            max_left_cols = max(self._shape(l)[1] for l in left)
-            max_left_rows = max(self._shape(l)[0] for l in left)
-            for i, (l, label) in enumerate(zip(left, labels)):
+            max_left_cols = max(self._shape(df)[1] for df in left)
+            max_left_rows = max(self._shape(df)[0] for df in left)
+            for i, (_left, _label) in enumerate(zip(left, labels)):
                 ax = fig.add_subplot(gs[i, 0:max_left_cols])
-                self._make_table(ax, l, title=label, height=1.0 / max_left_rows)
+                self._make_table(ax, _left, title=_label, height=1.0 / max_left_rows)
             # right
             ax = plt.subplot(gs[:, max_left_cols:])
             self._make_table(ax, right, title="Result", height=1.05 / vcells)
@@ -83,10 +90,10 @@ class TablePlotter:
             gs = gridspec.GridSpec(1, hcells)
             # left
             i = 0
-            for l, label in zip(left, labels):
-                sp = self._shape(l)
+            for df, _label in zip(left, labels):
+                sp = self._shape(df)
                 ax = fig.add_subplot(gs[0, i : i + sp[1]])
-                self._make_table(ax, l, title=label, height=height)
+                self._make_table(ax, df, title=_label, height=height)
                 i += sp[1]
             # right
             ax = plt.subplot(gs[0, i:])
@@ -96,7 +103,9 @@ class TablePlotter:
         return fig
 
     def _conv(self, data):
-        """Convert each input to appropriate for table outplot"""
+        """
+        Convert each input to appropriate for table outplot.
+        """
         if isinstance(data, pd.Series):
             if data.name is None:
                 data = data.to_frame(name="")
@@ -113,13 +122,13 @@ class TablePlotter:
             data.insert(0, "Index", data.index)
         else:
             for i in range(idx_nlevels):
-                data.insert(i, "Index{0}".format(i), data.index._get_level_values(i))
+                data.insert(i, f"Index{i}", data.index._get_level_values(i))
 
         col_nlevels = data.columns.nlevels
         if col_nlevels > 1:
             col = data.columns._get_level_values(0)
             values = [
-                data.columns._get_level_values(i).values for i in range(1, col_nlevels)
+                data.columns._get_level_values(i)._values for i in range(1, col_nlevels)
             ]
             col_df = pd.DataFrame(values)
             data.columns = col_df.columns
@@ -127,12 +136,12 @@ class TablePlotter:
             data.columns = col
         return data
 
-    def _make_table(self, ax, df, title, height=None):
+    def _make_table(self, ax, df, title: str, height: float | None = None) -> None:
         if df is None:
             ax.set_visible(False)
             return
 
-        import pandas.plotting as plotting
+        from pandas import plotting
 
         idx_nlevels = df.index.nlevels
         col_nlevels = df.columns.nlevels
@@ -158,7 +167,7 @@ class TablePlotter:
         ax.axis("off")
 
 
-if __name__ == "__main__":
+def main() -> None:
     import matplotlib.pyplot as plt
 
     p = TablePlotter()
@@ -179,8 +188,12 @@ if __name__ == "__main__":
     idx = pd.MultiIndex.from_tuples(
         [(1, "A"), (1, "B"), (1, "C"), (2, "A"), (2, "B"), (2, "C")]
     )
-    col = pd.MultiIndex.from_tuples([(1, "A"), (1, "B")])
+    column = pd.MultiIndex.from_tuples([(1, "A"), (1, "B")])
     df3 = pd.DataFrame({"v1": [1, 2, 3, 4, 5, 6], "v2": [5, 6, 7, 8, 9, 10]}, index=idx)
-    df3.columns = col
+    df3.columns = column
     p.plot(df3, df3, labels=["df3"])
     plt.show()
+
+
+if __name__ == "__main__":
+    main()
